/******************************************************************************/
/* idt.c  -- see http://www.zeppoo.net                                        */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "idt.h"

void getIdtMemory(pTable *myidt, char *file){
	zeppoo_get_idt(myidt);
	zeppoo_get_idt_opcodes(myidt);
	if(file == NULL)
		zeppoo_resolve_idt(myidt, DEFAULTSYSTEMMAP);
	else
		zeppoo_resolve_idt(myidt, file);
}

void getIdtFingerprints(FILE *input, pTable *myidt){
	char key[KEYSIZE];
	char line[80];
	char *pos, *stub_addr, *name, *op;
	pDescIdt *tmp_didt;
	
	fgets(line, 80, input);
	while(strcmp(line,"[END IDT]\n")){
		pos = strtok(line, " ");
		stub_addr = strtok(NULL, " ");
		name = strtok(NULL, " ");
		op = strtok(NULL, " ");	
		
		tmp_didt = (pDescIdt *)malloc(sizeof(pDescIdt));
		if(tmp_didt == NULL)
			zeppoo_fatal("malloc error");
		
		tmp_didt->pos = atoi(pos);
		tmp_didt->stub_addr = strtoul(stub_addr, NULL, 16);
		memcpy(tmp_didt->name, name, sizeof(tmp_didt->name));
		tmp_didt->opcodes[0] = strtoul(op, NULL, 16);
		op = strtok(NULL, " ");
		tmp_didt->opcodes[1] = strtoul(op, NULL, 16);

		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%d", tmp_didt->pos);
		hash_insert(myidt, key, KEYSIZE, tmp_didt);
	
		fgets(line, 80, input);
	}
}

void simpleViewIdt(pTable *myidt){
        pItem *tmp;
        pDescIdt *tmpdidt;
        printf("POS\t\tMEM\t\t\t NAME\t\t\t   OPCODES\n");

        tmp = myidt->first;
        while(tmp != NULL){ 
                tmpdidt = (pDescIdt *)tmp->el->value;
		printf("%d\t 0x%.8x\t %20s\t\t %8x %8x\n", tmpdidt->pos, (unsigned int)tmpdidt->stub_addr, tmpdidt->name, (unsigned int)tmpdidt->opcodes[0], (unsigned int)tmpdidt->opcodes[1]);
		 
                tmp = tmp->next;
        }
	
}

void viewIdtMemory(char *file){
	pTable *idtmemory;
	idtmemory = hash_new((void *)free_didt);

	getIdtMemory(idtmemory, file);
	simpleViewIdt(idtmemory);	
	
	hash_delete(idtmemory);
}

void writeIdtMemory(FILE *output, char *file){
	pTable *idtmemory;
	pItem *tmp;
	pDescIdt *tmpdidt;
					      
	idtmemory = hash_new((void *)free_didt);

	getIdtMemory(idtmemory, file);
	
	printf("\t++ Begin : Generating IDT Fingerprints\n");
	
	tmp = idtmemory->first;
	while(tmp != NULL){
		tmpdidt = (pDescIdt *)tmp->el->value;
		fprintf(output,"%d 0x%x %s %8x %8x\n", tmpdidt->pos, (unsigned int)tmpdidt->stub_addr, tmpdidt->name, (unsigned int)tmpdidt->opcodes[0], (unsigned int)tmpdidt->opcodes[1]);
	
		tmp = tmp->next;
	}			

	printf("\t++ End : Generating IDT Fingerprints\n\n");
	
	hash_delete(idtmemory);
}

void viewHijackIdt(FILE *input, char *file){
	char key[KEYSIZE];
	pTable *idtmemory = NULL;
	pTable *idtfingerprints = NULL;
	pTable *hijackidt = NULL;
	pItem *tmp;   
	pDescIdt *didtmem, *didtfing, *hijackdidt;
	
	pElement *hdidt;
	
	idtmemory = hash_new((void *)free_didt);
	idtfingerprints = hash_new((void *)free_didt);
	hijackidt = hash_new((void *)free_didt);
	
	getIdtMemory(idtmemory, file);
	getIdtFingerprints(input, idtfingerprints);

	tmp = idtfingerprints->first;
	while(tmp != NULL){
		bzero(key, KEYSIZE);
		didtfing = (pDescIdt *)tmp->el->value;	
		snprintf(key, KEYSIZE - 1, "%d", didtfing->pos);
		
		hdidt = (pElement *)hash_get(idtmemory, key, KEYSIZE);
		didtmem = (pDescIdt *)hdidt->value;

		if((didtfing->stub_addr != didtmem->stub_addr) || (didtfing->opcodes[0] != didtmem->opcodes[0]) || (didtfing->opcodes[1] != didtmem->opcodes[1])){           
			hijackdidt = (pDescIdt *)malloc(sizeof(pDescIdt));
			if(hijackdidt == NULL)
				zeppoo_fatal("malloc error");
			
			memcpy(hijackdidt->name, didtfing->name, sizeof(hijackdidt->name));
			hijackdidt->pos = didtfing->pos;
			hijackdidt->stub_addr = didtfing->stub_addr;
			hijackdidt->opcodes[0] = didtfing->opcodes[0];
			hijackdidt->opcodes[1] = didtfing->opcodes[1];
			hash_insert(hijackidt, key, KEYSIZE, hijackdidt);
		}
		tmp = tmp->next;
	}
	printf("-------------------------------------------------------------------------------\n");
	printf("[+] Begin : IDT\n\n");
	
	if(hijackidt->first != NULL){
		printf("LIST OF HIJACK IDT\n");
		simpleViewIdt(hijackidt);
		printf("\n");
	}
	else
		printf("NO HIJACK IDT\n\n");

	printf("[+] End : IDT\n");
	printf("-------------------------------------------------------------------------------\n\n");
	
	hash_delete(idtmemory);
	hash_delete(idtfingerprints);
	hash_delete(hijackidt);
}
