/******************************************************************************/
/* idt.c  -- see http://www.zeppoo.net                                        */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

void free_didt(pDescIdt *tmp){
	        free(tmp);
}

unsigned long get_addr_idt(void){
	asm("sidt %0" : "=m" (idtr));
	return idtr.base;        
}

unsigned short get_size_idt(void){
	asm("sidt %0" : "=m" (idtr));
	return idtr.limit;
}

int zeppoo_get_idt(pTable *myidt){
	unsigned long idt_table;
	unsigned short idt_size;
	int i;
	char key[KEYSIZE];
	pDescIdt *tmpdidt;
	
	idt_table = get_addr_idt();
	idt_size = get_size_idt();

#ifdef _DEBUG_
	printf("IDT TABLE : 0x%x , SIZE : %d\n", (unsigned int)idt_table, idt_size);
#endif
	bzero(key, KEYSIZE);
	
	for(i = 0;i < (idt_size + 1)/8; i++){
		tmpdidt = (pDescIdt *)malloc(sizeof(pDescIdt));
		if(tmpdidt == NULL){
			perror("malloc :");
			exit(-1);
		}
		zeppoo_read_memory(idt_table+8*i, &idt, sizeof(idt));
		tmpdidt->pos = i;
		tmpdidt->stub_addr = (unsigned long)(idt.off2 << 16) + idt.off1;	
		snprintf(key, KEYSIZE - 1, "%d", tmpdidt->pos);
		hash_insert(myidt, key, KEYSIZE, tmpdidt);
		bzero(key, KEYSIZE);
	}
	
	return 0;
}

int zeppoo_resolve_idt(pTable *myidt, char *file){
	pItem *tmp;
        pDescIdt *tmpdidt;

	tmp = myidt->first;
	while(tmp != NULL){
		tmpdidt = tmp->el->value;
		resolve(file, tmpdidt->stub_addr, tmpdidt->name, sizeof(tmpdidt->name));
		tmp = tmp->next;
	}

	return 0;
}

void zeppoo_get_idt_opcodes(pTable *myidt){
	pItem *tmp;
	pDescIdt *tmpdidt;

	tmp = myidt->first;
	while(tmp != NULL){
		tmpdidt = tmp->el->value;
		getOpcodes(tmpdidt->stub_addr, tmpdidt->opcodes);
		tmp = tmp->next;
	}
}
