/******************************************************************************/
/* memory.c  -- see http://www.zeppoo.net                                     */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

void zeppoo_init_memory(char *device, int val){
	valmmap = val;
	if(!strcmp(device, "/dev/kmem")){
		openvMem = openkmem;
		closevMem = closekmem;
		readvMem = readkmem;
		writevMem = writekmem; 

	}
	else if(!strcmp(device, "/dev/mem")){
		openvMem = openmem;
		closevMem = closemem;
		readvMem = readmem;
		writevMem = writemem;
	}
}

int zeppoo_open_memory(){
	int ret;
	ret = openvMem();
	return ret;
}

int zeppoo_close_memory(){
	int ret;
	ret = closevMem();
	return ret;
}

int zeppoo_read_memory(unsigned long offset, void *buf, int size){
	int ret;
	if(offset < PAGE_OFFSET || offset >= PAGE_MAX)
		return -1;

	ret = readvMem(offset, buf, size);
	return ret;
}

int zeppoo_fread_memory(unsigned long offset, void *buf, int size){
	int ret;
	int temp = valmmap;
	
	if(offset < PAGE_OFFSET || offset >= PAGE_MAX)
		return -1;
	
	valmmap = 0;
	ret  = readvMem(offset, buf, size);
	valmmap = temp;
	return ret;
}

int zeppoo_write_memory(unsigned long offset, void *buf, int size){
	int ret;
	
	if(offset < PAGE_OFFSET || offset >= PAGE_MAX)
		return -1;
	
	ret = writevMem(offset, buf, size);
	return ret;
}

int zeppoo_fwrite_memory(unsigned long offset, void *buf, int size){
	int ret;
	int temp = valmmap;
	
	if(offset < PAGE_OFFSET || offset >= PAGE_MAX)
		return -1;

	valmmap = 0;
	ret = writevMem(offset, buf, size);
	valmmap = temp;
	return ret;
}
