/******************************************************************************/
/* symbols.c  -- see http://www.zeppoo.net                                    */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

pKernelSym inittask_sym[] = { 
        { "proc_root_readdir", "proc_pid_readdir", 1, 1, 0xe9, 0, 0, 0 },
	{ "proc_pid_readdir", "get_tgid_list", 2, 1, 0xe8, 0, 0, 0 },
	{ "get_tgid_list", "init_task", 3, 1, 0x81, 0, 0, 0 },
	{ NULL, NULL, 0, 0, 0, 0, 0, 0 }
};

unsigned long lookup_root(){
        unsigned long proc_root;

	if(zepversion.kernel >= vers26)
		proc_root = lookup_root26();

	return proc_root;
}

/* Thanks to c0de @ UNF <c0de@uskf.com> for lookup_root */
/* Get @ lookup_root */
unsigned long lookup_root26(){
	int i;
	unsigned char buffer[4096];
	unsigned char proc_root[32];
  	unsigned long t = zepversion.kernel_start;
    
	while (t < zepversion.kernel_end) {
		zeppoo_read_memory(t, buffer, 4096);
      		for (i = 0; i < 4096; i++) {
			if(buffer[i] == PROC_ROOT_INO  && buffer[i+2] == PROC_ROOT_NOTHING && buffer[i+4] == PROC_ROOT_NAMELEN && buffer[i+12] == PROC_ROOT_MODE) {
					zeppoo_read_memory(t+i, proc_root, sizeof(proc_root));
				
					if(proc_root[16] == 0 && proc_root[20] == 0)
						return t+i;
			}
		}
		t+=4096;
	}
	return 0;
}

unsigned long zeppoo_find_init_task(void){
	unsigned long init_task;

	if(zepversion.kernel >= vers26)
		init_task = zeppoo_find_init_task26();

	return init_task;
}

/* Get @ init_task */
/* proc_root => proc_root_operations => proc_root_readdir => init_task */
unsigned long zeppoo_find_init_task26(void){
	float bound2615 = KERNEL2615;
	float bound2616 = KERNEL2616;
	
	unsigned long proc_root, proc_root_operations, proc_root_readdir, init_task;

	if(valmmap){
		init_task = walk_krstab("init_task", ptr, strlen ("init_task") + 2);
		if(validAddr(init_task) != 0){
			printf("[-] Unable to resolve init_task !!\n");
			exit(-1);
		}
#ifdef _DEBUG_
		printf("[+] INIT_TASK 0x%x\n", (unsigned int)init_task);
#endif
	}
	else{					
		proc_root = lookup_root();
		if(validAddr(proc_root) != 0){
			printf("[-] Unable to resolve proc_root !!\n");
			exit(-1);
		}
#ifdef _DEBUG_	
		printf("[+] proc_root @ 0x%x\n", (unsigned int)proc_root);
#endif
// < 2.6.15 + 32         == 2.6.15 + 36
		if(zepversion.kernel >= bound2615 && zepversion.kernel < bound2616)
			zeppoo_read_memory(proc_root+36, &proc_root_operations, 4);
		else
			zeppoo_read_memory(proc_root+32, &proc_root_operations, 4);
	
		if(validAddr(proc_root_operations) != 0){
			printf("[-] Unable to resolve proc_root_operations\n");
			exit(-1);
		}
#ifdef _DEBUG_
		printf("[+] proc_root_operations @ 0x%x\n",(unsigned int)proc_root_operations);
#endif

		zeppoo_read_memory(proc_root_operations+24, &proc_root_readdir, 4);

		if(validAddr(proc_root_readdir) != 0){
			printf("[-] Unable to resolve proc_root_readdir\n");
			exit(-1);
		}
#ifdef _DEBUG_
		printf("[+] proc_root_readdir @ 0x%x\n",(unsigned int)proc_root_readdir);
#endif	

		inittask_sym[0].start = proc_root_readdir;
		init_task = walk_tree(inittask_sym);
		
		if(validAddr(init_task) != 0){
			printf("[-] Unable to resolve init_task\n");
			exit(-1);
		}
	}

	return init_task;
}

int struc_size(pKernelSym *ksym) {
	int i;
	for (i = 0; ksym[i].caller; i++);
        return i;
}

unsigned long walk_tree(pKernelSym *ksym){
	int i, j, nb, ok, size;
	unsigned char buf[8192];
	unsigned long addr, t;
	pKernelSym *q;
	signed long offset;
	
	ok = 0;
	q = ksym;
	size = struc_size(ksym);
	t = q->start;

	for (i = 0; i < size; i++) {
		zeppoo_read_memory(t, buf, 8192);
		nb = 1;
		ok = 0;
		
		for (j = 0; j < 8192 && !ok; j++) {
			if (buf[j] == q->prefix){
				switch(q->prefix) {
					case 0xe9 : 
					case 0xe8 :
					zeppoo_read_memory(t + j + 1, &offset, sizeof(offset));
					addr = offset + 5 + t + j; 
					if(validAddr(addr) == 0) {
						q->address = addr;
						q->resolved = 1;
						getOpcodes(q->address, q->opcodes);
#ifdef _DEBUG_
						printf("\t[+] => %s 0x%x => %8x %8x\n", q->callee, (unsigned int)q->address, (unsigned int)q->opcodes[0], (unsigned int)q->opcodes[1]);
#endif	
						if(q->r == 1){
							t = addr;
							ok = 1;
						}
						q++;
					}
					break;
					
					case 0x81 :
					zeppoo_read_memory(t + j + 2, &addr, sizeof(addr));
					if(validAddr(addr) == 0) {
						q->address = addr;
						q->resolved = 1;
						getOpcodes(q->address, q->opcodes);
#ifdef _DEBUG_
						printf("\t[+] => %s 0x%x => %8x %8x\n", q->callee, (unsigned int)q->address, (unsigned int)q->opcodes[0], (unsigned int)q->opcodes[1]);
#endif
						if(q->r == 1){
							t = addr;
							ok = 1;
						}
						q++;
					}
					break;
				}
			}
		}
	}
	return addr;
}

/* Based on phalanx rootkit :) */
unsigned long walk_krstab(char *symbol, void *base, int size){
	char srch[512];
	unsigned long kstrtab, x, i = 0;
	srch[0] = '\0';
	memcpy (srch + 1, symbol, size);
	srch[size] = '\0';

	for (x = 0; x < 20 * 1024 * 1024; x++, i++){
#ifdef _DEBUG_
		if (i % 500000 == 0)
			write (1, "?", 1);
		else if (i == 4000001){
  	 		write (1, "\b\b\b\b\b\b\b\b        \b\b\b\b\b\b\b\b", 24);
			i = 0;
		}
#endif
      		if(memcmp ((unsigned char *) (base + x), srch, size) == 0){
	  		kstrtab = zepversion.kernel_start + x + 1;
	  	break;
		}
    	}

  	for (x = 0; x < 20 * 1024 * 1024; x++, i++){
#ifdef _DEBUG_
	if (i % 500000 == 0)
			write (1, "?", 1);
      	        else if(i == 4000001){
	  		write (1, "\b\b\b\b\b\b\b\b        \b\b\b\b\b\b\b\b", 24);
	  		i = 0;
		}
#endif
		if (*(unsigned long *) (base + x) == kstrtab)
			return *(unsigned long *) (base + x - 4);
    	}
	return 0;
}

void zeppoo_resolve_listsymbols(pKernelSyms *listKernelSyms){
	int i = 0;
	
	while(listKernelSyms[i].ksym != NULL){
		walk_tree(listKernelSyms[i].ksym);
		i++;
	}
}

void zeppoo_get_binfmt(pTask *mytask){
	if(mytask->mybin_fmt.format != 0){
		zeppoo_read_memory(mytask->mybin_fmt.format, &mytask->mybin_fmt.next, 4);
		zeppoo_read_memory(mytask->mybin_fmt.format+4, &mytask->mybin_fmt.module, 4);
		zeppoo_read_memory(mytask->mybin_fmt.format+8, &mytask->mybin_fmt.load_binary, 4);
		zeppoo_read_memory(mytask->mybin_fmt.format+12, &mytask->mybin_fmt.load_shlib, 4);
		zeppoo_read_memory(mytask->mybin_fmt.format+16, &mytask->mybin_fmt.core_dump, 4);
		getOpcodes(mytask->mybin_fmt.load_binary, &mytask->mybin_fmt.opcodes[0]);
		getOpcodes(mytask->mybin_fmt.load_shlib, &mytask->mybin_fmt.opcodes[2]);
		getOpcodes(mytask->mybin_fmt.core_dump, &mytask->mybin_fmt.opcodes[4]);
	}
}

void zeppoo_get_binfmts(pTable *linuxformats){
	char key[KEYSIZE];
	pTask ptask;
	unsigned long next;
	pLinuxBinfmt *mybinfmt;
	
	zeppoo_init_taskInfo();
	if(!zeppoo_get_task(1, &ptask)){
		mybinfmt = malloc(sizeof(pLinuxBinfmt));
		if(mybinfmt == NULL)
			zeppoo_fatal("malloc error");
		
		mybinfmt->format = ptask.mybin_fmt.format;
		mybinfmt->next = ptask.mybin_fmt.next;
		mybinfmt->module = ptask.mybin_fmt.module;
		mybinfmt->load_binary = ptask.mybin_fmt.load_binary;
		mybinfmt->load_shlib = ptask.mybin_fmt.load_shlib;
		mybinfmt->core_dump = ptask.mybin_fmt.core_dump;
	}

	bzero(key, KEYSIZE);
	snprintf(key, KEYSIZE - 1, "%x", (unsigned int)mybinfmt->format);
	hash_insert(linuxformats, key, KEYSIZE, mybinfmt);
	
	next = mybinfmt->next;
	while(next != 0){
		mybinfmt = malloc(sizeof(pLinuxBinfmt));
		if(mybinfmt == NULL)
			zeppoo_fatal("malloc error");
		
		mybinfmt->format = next;
		zeppoo_read_memory(mybinfmt->format, &mybinfmt->next, 4);
		zeppoo_read_memory(mybinfmt->format+4, &mybinfmt->module, 4);
		zeppoo_read_memory(mybinfmt->format+8, &mybinfmt->load_binary, 4);
		zeppoo_read_memory(mybinfmt->format+12, &mybinfmt->load_shlib, 4);
		zeppoo_read_memory(mybinfmt->format+16, &mybinfmt->core_dump, 4);
		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%x", (unsigned int)mybinfmt->format);
		hash_insert(linuxformats, key, KEYSIZE, mybinfmt);
		
		next = mybinfmt->next;
	}
}

void zeppoo_get_binfmts_opcodes(pTable *linuxformats){
	pItem *tmp;
	pLinuxBinfmt *tmpbinfmt;

	tmp = linuxformats->first;
	while(tmp != NULL){
		tmpbinfmt = (pLinuxBinfmt *)tmp->el->value;
		getOpcodes(tmpbinfmt->load_binary, &tmpbinfmt->opcodes[0]);
		getOpcodes(tmpbinfmt->load_shlib, &tmpbinfmt->opcodes[2]);
		getOpcodes(tmpbinfmt->core_dump, &tmpbinfmt->opcodes[4]);
		tmp = tmp->next;
	}
}

int zeppoo_resolve_binfmts(pTable *linuxformats, char *file){
	pItem *tmp;
        pLinuxBinfmt *tmpbinfmt;
	
	tmp = linuxformats->first;
        while(tmp != NULL){
		tmpbinfmt = (pLinuxBinfmt *)tmp->el->value;
		tmpbinfmt->name = malloc(sizeof(pNameLinuxBinfmt));
		if(tmpbinfmt->name == NULL)
			zeppoo_fatal("malloc error");

		resolve(file, tmpbinfmt->format, tmpbinfmt->name->name_format, sizeof(tmpbinfmt->name->name_format));
		resolve(file, tmpbinfmt->next, tmpbinfmt->name->name_next, sizeof(tmpbinfmt->name->name_next));
		resolve(file, tmpbinfmt->module, tmpbinfmt->name->name_module, sizeof(tmpbinfmt->name->name_module));
		resolve(file, tmpbinfmt->load_binary, tmpbinfmt->name->name_load_binary, sizeof(tmpbinfmt->name->name_load_binary));
		resolve(file, tmpbinfmt->load_shlib, tmpbinfmt->name->name_load_shlib, sizeof(tmpbinfmt->name->name_load_shlib));
		resolve(file, tmpbinfmt->core_dump, tmpbinfmt->name->name_core_dump, sizeof(tmpbinfmt->name->name_core_dump));

		tmp = tmp->next;
	}

	return 0;
}
