/******************************************************************************/
/* syscalls.c  -- see http://www.zeppoo.ne                                    */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

void free_syscall(pSyscall *tmp){
	free(tmp);
}

unsigned long zeppoo_get_syscall(int num){
	char buffer[255];
	unsigned long system_call, sys_call_table, addr;
	char *p;
	asm("sidt %0" : "=m" (idtr));

	zeppoo_read_memory(idtr.base+8*0x80, &idt, sizeof(idt));
	system_call = (idt.off2 << 16) | idt.off1;       
	zeppoo_read_memory(system_call, buffer, 255);
        p = (char *)memmem(buffer,255,"\xff\x14\x85",3);
	sys_call_table = *(unsigned long *)(p + 3);
	
	zeppoo_read_memory(sys_call_table + (4*num), &addr, 4);
	return addr;
}

int zeppoo_get_syscalls(pTable *mysyscalls){
	unsigned long system_call, sys_call_table;
	int i;
	char buffer[255];
	char key[KEYSIZE];
	char *p;
	pSyscall *tmp_syscall;

	asm("sidt %0" : "=m" (idtr));
#ifdef _DEBUG_
	printf("IDTR BASE %d 0x%x LIMIT 0x%x\n",idtr.base, idtr.base,idtr.limit);
#endif
	zeppoo_read_memory(idtr.base+8*0x80, &idt, sizeof(idt));


	system_call = (idt.off2 << 16) | idt.off1;
#ifdef _DEBUG_
	printf("SYSTEM_CALL : 0x%x\n", (unsigned int)system_call);
	printf("idt80: flags = %d flags=%X sel=%X off1=%x off2=%X system_call=%X\n",idt.flags,(unsigned)idt.flags,(unsigned)idt.sel,(unsigned)idt.off1, (unsigned)idt.off2, (unsigned int)system_call);
#endif
	zeppoo_read_memory(system_call, buffer, 255);

	p = (char *)memmem(buffer,255,"\xff\x14\x85",3);
	sys_call_table = *(unsigned long *)(p + 3);
#ifdef _DEBUG_
	printf("Sys Call Table 0x%x\n", (unsigned int)sys_call_table);
#endif
	for(i=0;i<320;i++){
		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%d", i);
		tmp_syscall = (pSyscall *)malloc(sizeof(pSyscall));
		if(tmp_syscall == NULL){
			perror("malloc :");
			exit(-1);
		}
		tmp_syscall->pos = i;
		zeppoo_read_memory(sys_call_table + (4*tmp_syscall->pos), &tmp_syscall->addr, 4);
		hash_insert(mysyscalls, key, KEYSIZE, tmp_syscall);
	}
	
	return 0;
}

int zeppoo_resolve_syscalls(pTable *mysyscalls, char *file){
	pItem *tmp;
        pSyscall *tmpsysc;

	tmp = mysyscalls->first;
	while(tmp != NULL){
		tmpsysc = tmp->el->value;
		resolve(file, tmpsysc->addr, tmpsysc->name, sizeof(tmpsysc->name));
		tmp = tmp->next;
	}

	return 0;
}

void zeppoo_get_syscalls_opcodes(pTable *mysyscalls){
	pItem *tmp;
	pSyscall *tmpsysc;

	tmp = mysyscalls->first;
	while(tmp != NULL){
		tmpsysc = (pSyscall *)tmp->el->value;
		getOpcodes(tmpsysc->addr, tmpsysc->opcodes);
		tmp = tmp->next;
	}
}
