/******************************************************************************/
/* tasks.c  -- see http://www.zeppoo.net                                      */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "zeppoo.h"

void free_task(pTask *tmp){
        free(tmp);
}

void zeppoo_init_taskInfo(void){
	vers26 = KERNEL26;
	vers24 = KERNEL24;

	if(zepversion.kernel >= vers26)
		zeppoo_init_taskInfo26();
}

void zeppoo_get_tasks(pTable *mytasks){
	if(zepversion.kernel >= vers26)
		zeppoo_get_tasks26(mytasks);
}

int find_offset_next(char *buffer, size_t size){
	int next = 0;
	
	if(zepversion.kernel >= vers26)
		next = find_offset_next26(buffer, size);
	
	return next;
}

int find_offset_name(char *buffer, size_t size){
	int name = 0;
	
	if(zepversion.kernel >= vers26)
		name = find_offset_name26(buffer, size);

	return name;
}

int find_offset_list(char *buffer, size_t size){
	int list = 0;
	
	if(zepversion.kernel >= vers26)
		list = find_offset_list26(buffer, size);

	return list;
}

int find_offset_binfmt(char *buffer, size_t size){
	int binfmt = 0;

	if(zepversion.kernel >= vers26)
		binfmt = find_offset_binfmt26(buffer, size);

	return binfmt;
}

int find_offset_pid(char *buffer, size_t size){
	int pid = 0;
	
	if(zepversion.kernel >= vers26)
		pid = find_offset_pid26(buffer, size);
	
	return pid;
}

int find_offset_uid(char *buffer, size_t size){
	int uid = 0;
	
	if(zepversion.kernel >= vers26)
		uid = find_offset_uid26(buffer, size);

	return uid;
}

/* Find offset to the next process */
int find_offset_next26(char *buffer, size_t size){
        int i,offset,find1 = 0,find2 = 0;
        unsigned long first_addr,second_addr;

        for(i=0;i<size;i+=4){
                second_addr = first_addr;
                first_addr = *(unsigned long *)(buffer+i);

                if((first_addr == second_addr) && (first_addr > ADDRMIN) && (i > 0x20)){
                        find2 = find1;
                        find1 = i;
 
                        if((find1-find2) == DIFFADDR){
#ifdef _DEBUG_                          
                                printf("[+] POS = 0x%x\n",find2+JMPFIVEADDR);
#endif
                                offset = find2+JMPFIVEADDR;
                                return offset;	              
			}
                }
        }
        return 0;
}

int find_offset_name26(char *buffer, size_t size){
	char *p;

	p = (char *)memmem(buffer, size, "\x73\x77\x61\x70\x70\x65\x72", 7);
	
	return (int)p - (int)buffer;
}

int find_offset_list26(char *buffer, size_t size){
	int i, count, offset;
	unsigned long first_addr, second_addr;
	count = 0;
	offset = -1;
	second_addr = -1;
	
	for(i=0;i<size && offset == -1;i+=4){
		first_addr = *(unsigned long *)(buffer+i);
		if(first_addr == second_addr)
			count++;
		if(count == 2){
			offset = i;
		}
		if(first_addr != 0)
			second_addr = first_addr;
	}
	return offset;
}

int find_offset_binfmt26(char *buffer, size_t size){
	int i, j, offset;
	unsigned long first_addr, second_addr;

	offset = first_addr = second_addr = -1;
	
	for(i=0, j=0;i<size && j != 3; i+=4){
		second_addr = first_addr;
		
		first_addr = *(unsigned long *)(buffer + i);
		if(first_addr == second_addr){
			j++;
			offset = i;
		}
	}
	return offset+4;
}

int find_offset_pid26(char *buffer, size_t size){
	int i, offset;
	unsigned long first_addr, second_addr;
	
	
	offset = first_addr = second_addr = -1;

	for(i=0;i<size && offset == -1;i+=4){
		if(first_addr == 1)
			second_addr = first_addr;
		
		first_addr = *(unsigned long *)(buffer + i);
		if(first_addr == second_addr)
			offset = i;
	}

	return offset;
}

int find_offset_uid26(char *buffer, size_t size){
	int offset, offset2;
	char *p;
	
	p = (char *)memmem(buffer, size, "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 28);

	offset = (int)p - (int)buffer;

	for(offset2 = 300; offset2<size; offset2+=4){
		p = (char *)memmem(buffer+offset2, size-offset2, "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00", 28);
		if(p == NULL)
			break;
	}

	if(offset2 <= 400)
		offset = offset2 - 8;
	
	return offset;
}

void zeppoo_init_taskInfo26(void){
	char buffer[1024];
        unsigned long list_addr, current_addr;

        zeptaskinfo.init_task = zeppoo_find_init_task();
#ifdef _DEBUG_
        printf("[+] GETTASKS INIT TASK @ 0x%x\n", (unsigned int)zeptaskinfo.init_task);
#endif

	zeppoo_fread_memory(zeptaskinfo.init_task, buffer, 1024);
        zeptaskinfo.offset_name = find_offset_name(buffer, 1024);
#ifdef _DEBUG_  
        printf("[+] OFFSET NAME %d\n", zeptaskinfo.offset_name);
#endif

        zeppoo_fread_memory(zeptaskinfo.init_task+40, buffer, 100);
        zeptaskinfo.offset_list = find_offset_list(buffer, 100) + 20;
#ifdef _DEBUG_
        printf("[+] OFFSET LIST %d\n", zeptaskinfo.offset_list);
#endif

        zeppoo_fread_memory(zeptaskinfo.init_task+zeptaskinfo.offset_list, &list_addr, 4);
        zeppoo_fread_memory(list_addr, buffer, 256);

        zeptaskinfo.offset_next = find_offset_next(buffer, 256);
#ifdef _DEBUG_
        printf("[+] OFFSET NEXT %d\n", zeptaskinfo.offset_next);
#endif

	zeppoo_fread_memory(list_addr + zeptaskinfo.offset_next, &current_addr, 4);
	zeppoo_fread_memory(current_addr + zeptaskinfo.offset_list, buffer, 100);
	
	zeptaskinfo.offset_binfmt = find_offset_binfmt(buffer, 100);
	zeptaskinfo.offset_binfmt += zeptaskinfo.offset_list;
#ifdef _DEBUG_
	printf("[+] OFFSET BINFMT %d\n", zeptaskinfo.offset_binfmt);
#endif

	zeptaskinfo.offset_pid = find_offset_pid(buffer, 100);
        zeptaskinfo.offset_pid += zeptaskinfo.offset_list;
#ifdef _DEBUG_
        printf("[+] OFFSET PID %d\n", zeptaskinfo.offset_pid);
#endif

        zeppoo_fread_memory(current_addr+zeptaskinfo.offset_pid, buffer, 500);
        zeptaskinfo.offset_uid = find_offset_uid(buffer, 500);
        zeptaskinfo.offset_uid += zeptaskinfo.offset_pid;
#ifdef _DEBUG_
        printf("[+] OFFSET UID %d\n", zeptaskinfo.offset_uid);
#endif
	zeptaskinfo.first_addr = current_addr;
}

void zeppoo_get_tasks26(pTable *mytasks){
        char name[16];
        char key[KEYSIZE];
        unsigned long bin_fmt, list_addr, current_addr;
        int pid, uid, gid;
        pTask *current_task;

	  
        bzero(key, KEYSIZE);

	current_addr = zeptaskinfo.first_addr;
	do{
                zeppoo_fread_memory(current_addr + zeptaskinfo.offset_name, name, 16);
                zeppoo_fread_memory(current_addr + zeptaskinfo.offset_binfmt, &bin_fmt, 4);
		zeppoo_fread_memory(current_addr + zeptaskinfo.offset_pid, &pid, 4);
                zeppoo_fread_memory(current_addr + zeptaskinfo.offset_uid, &uid, 4);
                zeppoo_fread_memory(current_addr + zeptaskinfo.offset_uid+16, &gid, 4);

                current_task = malloc(sizeof(pTask));
		if(current_task == NULL)
			zeppoo_fatal("malloc error");
                
		snprintf(key, KEYSIZE - 1, "%d",pid);
		bzero(current_task->name, sizeof(current_task->name));
                memcpy(current_task->name, name, sizeof(current_task->name));
		
		current_task->mybin_fmt.format = bin_fmt;
		current_task->pid = pid;
                current_task->uid = uid;
                current_task->gid = gid;
                current_task->addr = current_addr;

		zeppoo_get_binfmt(current_task);
                hash_insert(mytasks, key, KEYSIZE, current_task);

                bzero(key, KEYSIZE);
                zeppoo_fread_memory(current_addr + zeptaskinfo.offset_list, &list_addr, 4);
                zeppoo_fread_memory(list_addr + zeptaskinfo.offset_next, &current_addr, 4);
        }while(current_addr != zeptaskinfo.init_task);	
}

int zeppoo_get_task(int pid, pTask *rtask){
	char key[KEYSIZE];
	
	pTable *tasks;
	pElement *htask;
	pTask *tmptask;
	int ret = -1;


	bzero(key, KEYSIZE);
	tasks = hash_new((void *)free_task);
	zeppoo_get_tasks(tasks);

	snprintf(key, KEYSIZE - 1, "%d", pid);
	htask =  (pElement *)hash_get(tasks, key, KEYSIZE);
	
	if(htask != NULL){
		tmptask = (pTask *)htask->value;
		memcpy(rtask->name, tmptask->name, sizeof(rtask->name));
		rtask->pid = tmptask->pid;
		rtask->uid = tmptask->uid;
		rtask->gid = tmptask->gid;
		rtask->addr = tmptask->addr;
		rtask->mybin_fmt.format = tmptask->mybin_fmt.format;
		zeppoo_get_binfmt(rtask);
		ret = 0;
	}
	
	hash_delete(tasks);
	return ret;
}
