/******************************************************************************/
/* zeppoo.h  -- see http://www.zeppoo.net                                     */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#ifndef ZEPPOO_H
#define ZEPPOO_H

#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>


/******************* GLOBAL **************************/

#define DEFAULTSYSTEMMAP "/boot/System.map"
#define VERSION "0.0.3d"

struct {
	unsigned short limit;
	unsigned int base;
} __attribute__((packed)) idtr;

struct {
	unsigned short off1;
	unsigned short sel;
	unsigned char none,flags;
	unsigned short off2;
} __attribute__ ((packed)) idt;

typedef struct LinuxBinfmt pLinuxBinfmt;
typedef struct NameLinuxBinfmt pNameLinuxBinfmt;

struct LinuxBinfmt{
        unsigned long format;
	unsigned long next;
	unsigned long module;
	unsigned long load_binary;
	unsigned long load_shlib;
	unsigned long core_dump;
	unsigned long opcodes[6];	
	pNameLinuxBinfmt *name;
};

struct NameLinuxBinfmt{
        char name_format[64];
        char name_next[64];
	char name_module[64];
	char name_load_binary[64];
        char name_load_shlib[64];
        char name_core_dump[64];	
};

void getOpcodes(unsigned long, unsigned long *);
int resolve(char *, unsigned long, char *, int);
int validAddr(unsigned long);
void zeppoo_fatal(char *, ...);

/******************* HASH ****************************/

struct Element{
	unsigned long key;
	void *value;
};

struct Item {
	struct Element *el;
	struct Item *next;
};

struct Table {
	struct Item *first, *last, *current;
	void (*free_value)(void *);
};

typedef struct Element pElement;
typedef struct Item pItem;
typedef struct Table pTable;

unsigned long fnv_32a_buf(const void *, const unsigned int,  unsigned long);

pTable *hash_new(void (*)(void *));
void hash_delete(pTable *);

void hash_insert(pTable *, const void *, size_t, void *);
void hash_remove(pTable *, const void *, size_t);

pElement *hash_get(pTable *, const void *, size_t);

/******************* VERSION ************************/

#define KERNEL26 2.600000
#define KERNEL24 2.400000

#define KERNEL2615 2.615000
#define KERNEL2616 2.616000

/* I386 */
#define KERNEL_I386_START    0xc0000000      
#define KERNEL_I386_END      0xc1000000

struct Version {
	float kernel;
	unsigned long kernel_start;     
	unsigned long kernel_end;
	char archi[5];
};

typedef struct Version pVersion;

pVersion zepversion;
float vers26, vers24;

void zeppoo_init_version(char *, char *);

/****************** MEMORY **************************/

int mem;
void *ptr;
int valmmap;

void zeppoo_init_memory(char *, int);

int zeppoo_open_memory();
int zeppoo_close_memory();
int zeppoo_read_memory(unsigned long, void *, int);
int zeppoo_fread_memory(unsigned long, void *, int);
int zeppoo_write_memory(unsigned long, void *, int);
int zeppoo_fwrite_memory(unsigned long, void *, int);


int (*openvMem)();
int (*closevMem)();
int (*readvMem)(unsigned long, void *, int);
int (*writevMem)(unsigned long, void *, int);

/******************* KMEM *************************/

int openkmem();
int closekmem();
int readkmem(unsigned long, void *, int);
int writekmem(unsigned long, void *, int);

/******************* MEM ************************/

#define PAGE_OFFSET 0xc0000000
#define PAGE_MAX    0xffffffff

int openmem();
int closemem();
int readmem(unsigned long, void *, int);
int writemem(unsigned long, void *, int);

/****************** TASKS ***********************/

#define ADDRMIN         0xc0000000
#define DIFFADDR        0x10
#define JMPFIVEADDR     0x14

#define KEYSIZE 10 

struct TaskInfo{
	unsigned long init_task;
	unsigned long first_addr;
	int offset_name;
	int offset_list;
	int offset_next;
	int offset_pid;
	int offset_uid;
	int offset_binfmt;
};

typedef struct TaskInfo pTaskInfo;

pTaskInfo zeptaskinfo;

struct Task{
	char name[16];
	int pid;
	int uid;
	int gid;
	int rec;
	unsigned long addr;
	pLinuxBinfmt mybin_fmt;
};

typedef struct Task pTask;

void free_task(pTask *);

int find_offset_next(char *, size_t);
int find_offset_name(char *, size_t);
int find_offset_list(char *, size_t);
int find_offset_binfmt(char *, size_t);
int find_offset_pid(char *, size_t);
int find_offset_uid(char *, size_t);

int find_offset_next26(char *, size_t);
int find_offset_name26(char *, size_t);
int find_offset_list26(char *, size_t);
int find_offset_binfmt26(char *, size_t);
int find_offset_pid26(char *, size_t);
int find_offset_uid26(char *, size_t);

void zeppoo_init_taskInfo(void);
void zeppoo_get_tasks(pTable *);
int zeppoo_get_task(int, pTask *);

void zeppoo_init_taskInfo26(void);
void zeppoo_get_tasks26(pTable *);

/********************* SYSCALLS *********************/

struct Syscall{
	char name[64];
	int pos;
	unsigned long addr;
	unsigned long opcodes[2];
};

typedef struct Syscall pSyscall;

void free_syscall(pSyscall *);

int zeppoo_get_syscalls(pTable *);
int zeppoo_resolve_syscalls(pTable *, char *file);
unsigned long zeppoo_get_syscall(int);
void zeppoo_get_syscalls_opcodes(pTable *);
void getOpcodes(unsigned long , unsigned long *);

/******************** IDT ****************************/

struct DescIdt{
	char name[64];
	int pos;
	unsigned long stub_addr;
	unsigned long opcodes[2];
};

typedef struct DescIdt pDescIdt;

void free_didt(pDescIdt *);

int zeppoo_get_idt(pTable *);
int zeppoo_resolve_idt(pTable *, char *file);
void zeppoo_get_idt_opcodes(pTable *);


/****************** SYMBOLS *********************/

#define PROC_ROOT_INO           0x01
#define PROC_ROOT_NOTHING       0x00
#define PROC_ROOT_NAMELEN       0x05
#define PROC_ROOT_MODE          0x6d

typedef struct SymbolInfo pSymbolInfo;
typedef struct KernelSym pKernelSym;
typedef struct KernelSyms pKernelSyms;

struct SymbolInfo{
        unsigned long system_call;
        unsigned long sys_call_table;
};


pSymbolInfo zepsymbolinfo;

struct KernelSym{
	char *caller;
	char *callee;
	int number;
	int r;
	unsigned char prefix;
	unsigned long start;
	unsigned long address;
	unsigned long naddress;
	
	int resolved;
	unsigned long opcodes[2];
	unsigned long nopcodes[2];
	
	int hijack;
};

struct KernelSyms{
	char *name;
	int pos;
	int hijack;
	pKernelSym *ksym;
};

unsigned long zeppoo_find_init_task26(void);
unsigned long lookup_root26(void);

unsigned long zeppoo_find_init_task(void);
unsigned long lookup_root(void);

unsigned long walk_tree(pKernelSym *);
unsigned long walk_krstab(char *, void *, int);
void zeppoo_resolve_listsymbols(pKernelSyms []);

void zeppoo_get_binfmt(pTask *);
void zeppoo_get_binfmts(pTable *);
void zeppoo_get_binfmts_opcodes(pTable *);
int zeppoo_resolve_binfmts(pTable *, char *);

#endif
