/******************************************************************************/
/* main.c  -- see http://www.zeppoo.net                                       */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>

#include "libzeppoo/zeppoo.h"
#include "tasks.h"
#include "syscalls.h"
#include "idt.h"
#include "symbols.h"
#include "fingerprints.h"

void help(char *name){
	printf("HELP :\n");
	printf("-z FP\t check the system !\n"); 
	printf("-p\t display tasks in memory\n");
	printf("-s\t display syscalls\n");
	printf("-i\t display idt\n");
	printf("-f FILE\t generate a fingerprint(syscalls, idt)\n");
	printf("-c OPTIONS\t check tasks, networks, fingerprints\n");
	printf("-d DEVICE\t use device(/dev/mem, /dev/kmem)\n");
	printf("-m\t\t more quick/portable\n");
	printf("-t SYSTEMMAP\t specify System.map\n");
	printf("-v\t\t version\n");
	exit(0);
}

int main(int argc, char *argv[]){
	char c;
	int task,syscall,idt,symbol,network,finger,check,device,kernel,archi,ommap,map,all;
	char *archiarg, *kernelarg, *devicearg, *taskarg, *maparg, *fingerarg, *allarg;

	task = syscall = idt = symbol = network = finger = check = device = kernel = archi = ommap = map = all = 0;
	
	archiarg = kernelarg = devicearg = taskarg = maparg = fingerarg = allarg = NULL;
	
	if(getuid() != 0){
		fprintf(stderr, "You must be root !!\n");
		exit(-1);
	}
	
	while((c = getopt(argc, argv, "hz:vp::sxnf:ict:d:a:k:m")) != -1){
		switch(c){		
			case 'h' :
				help(argv[0]);
				break;
			case 'z' :
				all++;
				allarg = optarg;
				break;
			case 'v' :
				printf("VERSION %s\n", VERSION);
				exit(0);
				break;
			case 'p':
				task++;
				if(optarg != NULL)
					taskarg = optarg+1;
				break;
			case 's' :
				syscall++;
				break;
			case 'i' :
				idt++;
				break;
			case 'x' :
				symbol++;
				break;
			case 'n' :
				network++;
				break;
			case 't' :
				map++;
				maparg = optarg;
				break;
			case 'f' :
				finger++;
				fingerarg = optarg;
				break;
			case 'c' :
				check++;
				break;
			case 'd' :
				device++;
				devicearg = optarg;
				break;
			case 'a' :
				archi++;
				archiarg = optarg;
				break;
			case 'k' :
				kernel++;
				kernelarg = optarg;
				break;
			case 'm' :
				ommap++;
				break;
		}
	}

	if(task || syscall || idt || symbol || network || finger || check || device || kernel || archi || ommap || map || all){
		zeppoo_init_version(kernelarg, archiarg);
	
		if(devicearg)
			zeppoo_init_memory(devicearg, ommap);
		else
			zeppoo_init_memory("/dev/kmem", ommap);
	
		zeppoo_open_memory();

		if(all){
			viewHiddenTasks();
			checkFingerprints(allarg, maparg);
		}
		else if(check){
			if(task)
				viewHiddenTasks();
			else if(finger)
				checkFingerprints(fingerarg, maparg);
		}
		else if(finger){
			doFingerprints(fingerarg, maparg);
		}
		else if(task){
			if(taskarg != NULL)
				viewTaskMemory(atoi(taskarg));
			else
				viewTasksMemory();
		}
		else if(syscall){
			viewSyscallsMemory(maparg);
		}
		else if(idt){
			viewIdtMemory(maparg);
		}
		else if(symbol){
			viewSymbols(maparg);
			viewSymbolsLinuxBinfmt(maparg);
		}
		else if(network){
		}
		
		zeppoo_close_memory();
	}
	else{
		printf("Help : -h\n");
	}

	return 0;
}
