/******************************************************************************/
/* symbols.c  -- see http://www.zeppoo.net                                    */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "symbols.h"

pKernelSym execve_sym[] = {
	{ "sys_execve", "getname", 1, 0, 0xe8, 0, 0, 0, 0, { 0, 0 }, { 0, 0 } },
	{ "sys_execve", "do_execve", 2, 1, 0xe8, 0, 0, 0, 0, { 0, 0 }, { 0, 0 } },
	{ "do_execve", "kmem_cache_alloc", 3, 0, 0xe8, 0, 0, 0, 0, { 0, 0 }, { 0, 0 } },
	{ "do_execve", "open_exec", 4, 0, 0xe8, 0, 0, 0, 0, { 0, 0 }, { 0, 0 } },
	{ NULL, NULL, 0, 0, 0, 0, 0, 0, 0, { 0, 0 }, { 0, 0 } }
};

pKernelSyms listKernelSyms[] = {
	{ "execve", __NR_execve, 0, execve_sym },
	{ NULL, -1, 0, NULL }
};

void free_binfmt(pLinuxBinfmt *tmp){
	if(tmp->name != NULL)
		free(tmp->name);

	free(tmp);
}

void simpleViewSymbol(pKernelSym *ksym, int type){
	printf("\t[+] => %s ", ksym->callee);

	if(type == 0)
		printf("0x%x => %8x %8x\n", (unsigned int)ksym->address, (unsigned int)ksym->opcodes[0], (unsigned int)ksym->opcodes[1]);
	else
		printf("0x%x => %8x %8x\n", (unsigned int)ksym->naddress, (unsigned int)ksym->nopcodes[0], (unsigned int)ksym->nopcodes[1]);
}

void simpleViewSymbols(pKernelSyms *listKernelSyms){
	int i,j;

	i = j = 0;
	while(listKernelSyms[i].ksym != NULL){
		printf("[+] %s\n", listKernelSyms[i].name);
		j = 0;
		while(listKernelSyms[i].ksym[j].callee != NULL){
			simpleViewSymbol(&listKernelSyms[i].ksym[j], 0);
			j++;
		}

		i++;
	}

}

void getSymbols(pKernelSyms *listKernelSyms){
	int i = 0;
	/* Init Start Adress */
	while(listKernelSyms[i].ksym != NULL){
		if(listKernelSyms[i].pos != -1)
			listKernelSyms[i].ksym[0].start = zeppoo_get_syscall(listKernelSyms[i].pos);
		i++;
	}
	zeppoo_resolve_listsymbols(listKernelSyms);
}

void viewSymbols(char *file){
	getSymbols(listKernelSyms);
	printf("[*] Symbols\n");
	simpleViewSymbols(listKernelSyms);
}

void writeSymbols(FILE *output, char *file){
	int i, j;
	i = j = 0;
	
	getSymbols(listKernelSyms);
	
	printf("\t++ Begin : Generating Symbols Fingerprints\n");
	while(listKernelSyms[i].ksym != NULL){
		j = 0;
		while(listKernelSyms[i].ksym[j].callee != NULL){
			fprintf(output, "%d %d %s 0x%x %x %x\n", i, listKernelSyms[i].ksym[j].number, listKernelSyms[i].name, (unsigned int)listKernelSyms[i].ksym[j].address, (unsigned int)listKernelSyms[i].ksym[j].opcodes[0], (unsigned int)listKernelSyms[i].ksym[j].opcodes[1]);
			j++;
		}
		i++;
	}
	printf("\t++ End : Generating Symbols Fingerprints\n\n");
}

void getSymbolFingerprints(pKernelSym *ksym){
	char *addr, *op;

	strtok(NULL, " ");

	addr = strtok(NULL, " ");
	ksym->naddress = strtoul(addr, NULL, 16);

	op = strtok(NULL, " ");
	ksym->nopcodes[0] = strtoul(op, NULL, 16);

	op = strtok(NULL, " ");
	ksym->nopcodes[1] = strtoul(op, NULL, 16);	
}

void getSymbolsFingerprints(FILE *input){
	int i, j;
	i = j = 0;
	int col, lin;

	char line[256];
	
	fgets(line, 256, input);
	while(strcmp(line, "[END SYMBOLS]\n")){
		col = atoi(strtok(line, " "));
		lin = atoi(strtok(NULL, " "));

		getSymbolFingerprints(&listKernelSyms[col].ksym[lin-1]);
		
		fgets(line, 256, input);
	}
}

void viewHijackSymbols(FILE *input){
	int i, j, hijack;
	i = j = hijack = 0;
	
	getSymbols(listKernelSyms);
	getSymbolsFingerprints(input);

	printf("-------------------------------------------------------------------------------\n");
	printf("[+] Begin : Symbols\n\n");

	while(listKernelSyms[i].ksym != NULL){
		j = 0;
		while(listKernelSyms[i].ksym[j].callee != NULL){
			if(listKernelSyms[i].ksym[j].address != listKernelSyms[i].ksym[j].naddress || listKernelSyms[i].ksym[j].opcodes[0] != listKernelSyms[i].ksym[j].nopcodes[0] || listKernelSyms[i].ksym[j].opcodes[1] != listKernelSyms[i].ksym[j].nopcodes[1]){
				hijack = 1;
				listKernelSyms[i].ksym[j].hijack = 1;
				listKernelSyms[i].hijack = 1;
			}
			j++;
		}
		i++;
	}

	if(hijack == 1){
		printf("LIST OF HIJACK SYMBOLS\n");
		i = 0;
		while(listKernelSyms[i].ksym != NULL){
			if(listKernelSyms[i].hijack == 1){
				printf("[+] %s\n", listKernelSyms[i].name);
				j = 0;
				while(listKernelSyms[i].ksym[j].callee != NULL){
					if(listKernelSyms[i].ksym[j].hijack == 1){
						simpleViewSymbol(&listKernelSyms[i].ksym[j], 1);
					}
					j++;
				}
			}
			i++;
		}
		printf("\n");
	}
	else{
		printf("NO HIJACK SYMBOL\n\n");
	}

	printf("[+] End : Symbols\n\n");
	printf("-------------------------------------------------------------------------------\n\n");
}

void simpleViewLinuxBinfmt(pTable *linuxformats){
	pItem *tmp;
	pLinuxBinfmt *tmpbinfmt;
	
	tmp = linuxformats->first;
	while(tmp != NULL){
		tmpbinfmt = (pLinuxBinfmt *)tmp->el->value;

		printf("[+] format : %s => 0x%x\n", tmpbinfmt->name->name_format, (unsigned int)tmpbinfmt->format);
		printf("\t[+] next : %s => 0x%x\n", tmpbinfmt->name->name_next, (unsigned int)tmpbinfmt->next);
		printf("\t[+] module : %s => 0x%x\n", tmpbinfmt->name->name_module, (unsigned int)tmpbinfmt->module);
		printf("\t[+] load_binary : %s => 0x%x => %x %x\n", tmpbinfmt->name->name_load_binary, (unsigned int)tmpbinfmt->load_binary, (unsigned int)tmpbinfmt->opcodes[0], (unsigned int)tmpbinfmt->opcodes[1]);
		printf("\t[+] load_shlib : %s => 0x%x => %x %x\n", tmpbinfmt->name->name_load_shlib, (unsigned int)tmpbinfmt->load_shlib,  (unsigned int)tmpbinfmt->opcodes[2], (unsigned int)tmpbinfmt->opcodes[3]);
		printf("\t[+] core_dump : %s => 0x%x => %x %x\n", tmpbinfmt->name->name_core_dump, (unsigned int)tmpbinfmt->core_dump,  (unsigned int)tmpbinfmt->opcodes[4], (unsigned int)tmpbinfmt->opcodes[5]);	

		tmp = tmp->next;
	}
	printf("\n");
}

void getSymbolsLinuxBinfmt(pTable *linuxformats, char *file){
	zeppoo_get_binfmts(linuxformats);
	zeppoo_get_binfmts_opcodes(linuxformats);

	if(file == NULL)
		zeppoo_resolve_binfmts(linuxformats, DEFAULTSYSTEMMAP);
	else
		zeppoo_resolve_binfmts(linuxformats, file);
}

void viewSymbolsLinuxBinfmt(char *file){
	pTable *linuxformats = NULL;
		        
	linuxformats = hash_new((void *)free_binfmt);

	getSymbolsLinuxBinfmt(linuxformats, file);
	printf("[*] Symbols Linux Binfmt\n");
	simpleViewLinuxBinfmt(linuxformats);

	hash_delete(linuxformats);
}

void writeSymbolsLinuxBinfmt(FILE *output, char *file){
	pTable *linuxformats = NULL;
	pItem *tmp;
	pLinuxBinfmt *tmpbinfmt;

	linuxformats = hash_new((void *)free_binfmt);
	getSymbolsLinuxBinfmt(linuxformats, file);

	printf("\t++ Begin : Generating Symbols LinuxBinfmt Fingerprints\n");
	
	tmp = linuxformats->first;
	while(tmp != NULL){
		tmpbinfmt = (pLinuxBinfmt *)tmp->el->value;
		
		fprintf(output,"%s 0x%x ", tmpbinfmt->name->name_format, (unsigned int)tmpbinfmt->format);
		fprintf(output,"%s 0x%x ", tmpbinfmt->name->name_next, (unsigned int)tmpbinfmt->next);
		fprintf(output,"%s 0x%x ", tmpbinfmt->name->name_module, (unsigned int)tmpbinfmt->module);
		fprintf(output,"%s 0x%x %x %x ", tmpbinfmt->name->name_load_binary, (unsigned int)tmpbinfmt->load_binary, (unsigned int)tmpbinfmt->opcodes[0], (unsigned int)tmpbinfmt->opcodes[1]);
		fprintf(output,"%s 0x%x %x %x ", tmpbinfmt->name->name_load_shlib, (unsigned int)tmpbinfmt->load_shlib, (unsigned int)tmpbinfmt->opcodes[2], (unsigned int)tmpbinfmt->opcodes[3]);
		fprintf(output,"%s 0x%x %x %x\n", tmpbinfmt->name->name_core_dump, (unsigned int)tmpbinfmt->core_dump, (unsigned int)tmpbinfmt->opcodes[4], (unsigned int)tmpbinfmt->opcodes[5]);
		
		tmp = tmp->next;
	}

	printf("\t++ End : Generating Symbols LinuxBinfmt Fingerprints\n\n");
	hash_delete(linuxformats);
}

void copyLinuxBinfmt(pLinuxBinfmt *source, pLinuxBinfmt *dest){
	dest->format = source->format;
	dest->next = source->next;
	dest->module = source->module;
	dest->load_binary = source->load_binary;
	dest->load_shlib = source->load_shlib;
	dest->core_dump = source->core_dump;

	memcpy(dest->name->name_format, source->name->name_format, sizeof(dest->name->name_format) - 1);
	memcpy(dest->name->name_next, source->name->name_next, sizeof(dest->name->name_next) - 1);
	memcpy(dest->name->name_module, source->name->name_module, sizeof(dest->name->name_module) - 1);
	memcpy(dest->name->name_load_binary, source->name->name_load_binary, sizeof(dest->name->name_load_binary) - 1);
	memcpy(dest->name->name_load_shlib, source->name->name_load_shlib, sizeof(dest->name->name_load_shlib) - 1);
	memcpy(dest->name->name_core_dump, source->name->name_core_dump, sizeof(dest->name->name_core_dump) - 1);

}

void getLinuxBinfmt(unsigned long *value, char *name, unsigned long *tabopcodes, size_t size){
	char *champ;
	
	champ = strtok(NULL, " ");
	memcpy(name, champ, size - 1);
	champ = strtok(NULL, " ");
	*value = strtoul(champ, NULL, 16);

	if(tabopcodes != NULL){
		champ = strtok(NULL, " ");
		tabopcodes[0] = strtoul(champ, NULL, 16);
		champ = strtok(NULL, " ");
		tabopcodes[1] = strtoul(champ, NULL, 16);
	}
}

void getLinuxBinfmtFingerprints(FILE *input, pTable *linuxformats){
	char key[KEYSIZE];
	char line[256];
	char *champ;
	pLinuxBinfmt *tmpbinfmt;
	
	fgets(line, 256, input);
	while(strcmp(line, "[END SYMBOLS LINUX BINFMT]\n")){
		tmpbinfmt = (pLinuxBinfmt *)malloc(sizeof(pLinuxBinfmt));
		tmpbinfmt->name = (pNameLinuxBinfmt *)malloc(sizeof(pNameLinuxBinfmt));

		if(tmpbinfmt == NULL || tmpbinfmt->name == NULL)
			zeppoo_fatal("malloc error");
		
		champ = strtok(line, " ");
		memcpy(tmpbinfmt->name->name_format, champ, sizeof(tmpbinfmt->name->name_format) - 1);
		champ = strtok(NULL, " ");
		tmpbinfmt->format = strtoul(champ, NULL, 16);	
	
		getLinuxBinfmt(&tmpbinfmt->next, tmpbinfmt->name->name_next, NULL, sizeof(tmpbinfmt->name->name_next));
	
		getLinuxBinfmt(&tmpbinfmt->module, tmpbinfmt->name->name_module,NULL, sizeof(tmpbinfmt->name->name_module));
	
		getLinuxBinfmt(&tmpbinfmt->load_binary, tmpbinfmt->name->name_load_binary,  &tmpbinfmt->opcodes[0], sizeof(tmpbinfmt->name->name_load_binary));

		getLinuxBinfmt(&tmpbinfmt->load_shlib, tmpbinfmt->name->name_load_shlib, &tmpbinfmt->opcodes[2], sizeof(tmpbinfmt->name->name_load_shlib));

		getLinuxBinfmt(&tmpbinfmt->core_dump, tmpbinfmt->name->name_core_dump, &tmpbinfmt->opcodes[4], sizeof(tmpbinfmt->name->name_core_dump));
		
		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%x", (unsigned int)tmpbinfmt->format);
		hash_insert(linuxformats, key, KEYSIZE, tmpbinfmt);
		
		fgets(line, 256, input);
	}
}

pLinuxBinfmt *checkSymbolsLinuxBinfmtTask(pTask *tmptask, pTable *linuxformats){
	pItem *tmp;
        pLinuxBinfmt *tmpbinfmt;
	pLinuxBinfmt *hijackbinfmt;
	
	int format, hijack, ret;
	
	format = hijack = ret = 0;
	
	tmp = linuxformats->first;
	while(tmp != NULL && format == 0){
		tmpbinfmt = (pLinuxBinfmt *)tmp->el->value;
		if(tmptask->mybin_fmt.format == tmpbinfmt->format){
			format = 1;
			if(tmptask->mybin_fmt.next != tmpbinfmt->next)
				hijack++;
			if(tmptask->mybin_fmt.module != tmpbinfmt->module)
				hijack++;
			if(tmptask->mybin_fmt.load_binary != tmpbinfmt->load_binary || tmptask->mybin_fmt.opcodes[0] != tmpbinfmt->opcodes[0] || tmptask->mybin_fmt.opcodes[1] != tmpbinfmt->opcodes[1])
				hijack++;
			if(tmptask->mybin_fmt.load_shlib != tmpbinfmt->load_shlib || tmptask->mybin_fmt.opcodes[2] != tmpbinfmt->opcodes[2] || tmptask->mybin_fmt.opcodes[3] != tmpbinfmt->opcodes[3])
				hijack++;
			if(tmptask->mybin_fmt.core_dump != tmpbinfmt->core_dump || tmptask->mybin_fmt.opcodes[4] != tmpbinfmt->opcodes[4] || tmptask->mybin_fmt.opcodes[5] != tmpbinfmt->opcodes[5])
				hijack++;
		}
		tmp = tmp->next;
	}

	if(format == 1 && hijack == 0)
		hijackbinfmt = NULL;
	else{
		hijackbinfmt = (pLinuxBinfmt *)malloc(sizeof(pLinuxBinfmt));
		hijackbinfmt->name = (pNameLinuxBinfmt *)malloc(sizeof(pNameLinuxBinfmt));
		if(hijackbinfmt == NULL || hijackbinfmt->name == NULL)
			zeppoo_fatal("malloc error");
		
		copyLinuxBinfmt(tmpbinfmt, hijackbinfmt);

		return hijackbinfmt;
	}
	
	return hijackbinfmt;
}

void checkSymbolsLinuxBinfmt(pTable *linuxformats, pTable *check){
	char key[KEYSIZE];
	pItem *tmp;
        pTask *tmptask;
	pTable *tasksmemory;
	
	pLinuxBinfmt *hijackbinfmt;

	tasksmemory = hash_new((void *)free_task);
	zeppoo_init_taskInfo();
	zeppoo_get_tasks(tasksmemory);

	tmp = tasksmemory->first;
	while(tmp != NULL){
		tmptask = (pTask *)tmp->el->value;

		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%x", (unsigned int)tmptask->mybin_fmt.format);
		if(tmptask->mybin_fmt.format != 0 && hash_get(check, key, KEYSIZE) == NULL){	
			if((hijackbinfmt = checkSymbolsLinuxBinfmtTask(tmptask, linuxformats)) != NULL){
				bzero(key, KEYSIZE);
				snprintf(key, KEYSIZE - 1, "%x", (unsigned int)hijackbinfmt->format);
				hash_insert(check, key, KEYSIZE, hijackbinfmt);
			}
		}
		tmp = tmp->next;
	}
	
	hash_delete(tasksmemory);
}

void viewHijackSymbolsLinuxBinfmt(FILE *input, char *file){
	pTable *linuxformats = NULL;
	pTable *linuxformatscheck = NULL;
	
	linuxformats = hash_new((void *)free_binfmt);
	linuxformatscheck = hash_new((void *)free_binfmt);
	
	getLinuxBinfmtFingerprints(input, linuxformats);

	checkSymbolsLinuxBinfmt(linuxformats, linuxformatscheck);
	printf("-------------------------------------------------------------------------------\n");
	printf("[+] Begin : Symbols Linux Binfmt\n\n");
	if(linuxformatscheck->first != NULL){
		printf("LIST OF HIJACK SYMBOLS LINUX BINFMT\n");
		simpleViewLinuxBinfmt(linuxformatscheck);
		printf("\n");
	}
	else
		printf("NO HIJACK SYMBOL LINUX BINFMT\n\n");
	
	printf("[+] End : Symbols Linux Binfmt\n\n");
	printf("-------------------------------------------------------------------------------\n\n");
	
	hash_delete(linuxformats);
	hash_delete(linuxformatscheck);
}
