/******************************************************************************/
/* syscalls.c  -- see http://www.zeppoo.net                                   */
/*                                                                            */
/* The project zeppoo is (C) 2006 : contact@zeppoo.net                        */
/* This program is free software;                                             */
/* you can redistribute it and/or modify it under the terms of the GNU        */
/* General Public License as published by the Free Software Foundation;       */
/* Version 2. This guarantees your right to use, modify, and                  */
/* redistribute this software under certain conditions.                       */
/*                                                                            */
/* Source is provided to this software because we believe users have a        */
/* right to know exactly what a program is going to do before they run        */
/* it.                                                                        */
/*                                                                            */
/* This program is distributed in the hope that it will be                    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied                 */
/* warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                    */
/* PURPOSE. See the GNU General Public License for more details (             */
/* http://www.gnu.org/copyleft/gpl.html ).                                    */
/*                                                                            */
/******************************************************************************/

#include "syscalls.h"

void getSyscallsMemory(pTable *mysyscalls, char *file){
	zeppoo_get_syscalls(mysyscalls);
	zeppoo_get_syscalls_opcodes(mysyscalls);
	if(file == NULL)
		zeppoo_resolve_syscalls(mysyscalls, DEFAULTSYSTEMMAP);
	else 
		zeppoo_resolve_syscalls(mysyscalls, file);
}

void getSyscallsFingerprints(FILE *input, pTable *mysyscalls){
	char key[KEYSIZE];
	char line[80];
	char *pos, *addr, *name, *op;
	pSyscall *tmp_syscall;
	
	fgets(line, 80, input);
	while(strcmp(line,"[END SYSCALLS]\n")){
		pos = strtok(line, " ");
		addr = strtok(NULL, " ");
		name = strtok(NULL, " ");
		op = strtok(NULL, " ");	
		
		tmp_syscall = (pSyscall *)malloc(sizeof(pSyscall));
		if(tmp_syscall == NULL)
			zeppoo_fatal("malloc error");
		
		tmp_syscall->pos = atoi(pos);
		tmp_syscall->addr = strtoul(addr, NULL, 16);
		memcpy(tmp_syscall->name, name, sizeof(tmp_syscall->name) - 1);
		tmp_syscall->opcodes[0] = strtoul(op, NULL, 16);
		op = strtok(NULL, " ");
		tmp_syscall->opcodes[1] = strtoul(op, NULL, 16);

		bzero(key, KEYSIZE);
		snprintf(key, KEYSIZE - 1, "%d", tmp_syscall->pos);
		hash_insert(mysyscalls, key, KEYSIZE, tmp_syscall);
	
		fgets(line, 80, input);
	}
}

void simpleViewSyscalls(pTable *mysyscalls){
        pItem *tmp;
        pSyscall *tmpsysc;
        printf("POS\t\tMEM\t\t\t NAME\t\t\t   OPCODES\n");

        tmp = mysyscalls->first;
        while(tmp != NULL){ 
		tmpsysc = (pSyscall *)tmp->el->value;
                printf("%d\t 0x%.8x\t %20s\t\t %8x %8x\n", tmpsysc->pos, (unsigned int)tmpsysc->addr, tmpsysc->name, (unsigned int)tmpsysc->opcodes[0], (unsigned int)tmpsysc->opcodes[1]);
	       
		 
                tmp = tmp->next;
		
        }
}

void viewSyscallsMemory(char *file){
	pTable *syscallsmemory;
	syscallsmemory = hash_new((void *)free_syscall);

	getSyscallsMemory(syscallsmemory, file);
	simpleViewSyscalls(syscallsmemory);	
	
	hash_delete(syscallsmemory);
}

void writeSyscallsMemory(FILE *output, char *file){
	pTable *syscallsmemory;
	pItem *tmp;
	pSyscall *tmpsysc;
					      
	syscallsmemory = hash_new((void *)free_syscall);

	getSyscallsMemory(syscallsmemory, file);	

	printf("\t++ Begin : Generating Syscalls Fingerprints\n");
	
	tmp = syscallsmemory->first;
	while(tmp != NULL){
		tmpsysc = (pSyscall *)tmp->el->value;
		fprintf(output,"%d 0x%x %s %8x %8x\n", tmpsysc->pos, (unsigned int)tmpsysc->addr, tmpsysc->name, (unsigned int)tmpsysc->opcodes[0], (unsigned int)tmpsysc->opcodes[1]);
							                
		tmp = tmp->next;
	}					

	printf("\t++ End : Generating Syscalls Fingerprints\n\n");
	
	hash_delete(syscallsmemory);
}

void viewHijackSyscalls(FILE *input, char *file){
	char key[KEYSIZE];
	pTable *syscallsmemory = NULL;
	pTable *syscallsfingerprints = NULL;
	pTable *hijacksyscalls = NULL;
	pItem *tmp;   
	pSyscall *syscmem, *syscfing, *hijacksysc;
	
	pElement *hsyscall;
	
	syscallsmemory = hash_new((void *)free_syscall);
	syscallsfingerprints = hash_new((void *)free_syscall);
	hijacksyscalls = hash_new((void *)free_syscall);
	
	getSyscallsMemory(syscallsmemory, file);
	getSyscallsFingerprints(input, syscallsfingerprints);
	
	tmp = syscallsfingerprints->first;
	while(tmp != NULL){
		bzero(key, KEYSIZE);
		syscfing = (pSyscall *)tmp->el->value;	
		snprintf(key, KEYSIZE - 1, "%d", syscfing->pos);
		
		hsyscall = (pElement *)hash_get(syscallsmemory, key, KEYSIZE);
		syscmem = (pSyscall *)hsyscall->value;
	
		if((syscfing->addr != syscmem->addr) || (syscfing->opcodes[0] != syscmem->opcodes[0]) || (syscfing->opcodes[1] != syscmem->opcodes[1])){           
			hijacksysc = (pSyscall *)malloc(sizeof(pSyscall));
			if(hijacksysc == NULL)
				zeppoo_fatal("malloc error");
			
			memcpy(hijacksysc->name, syscfing->name, sizeof(hijacksysc->name));
			hijacksysc->pos = syscfing->pos;
			hijacksysc->addr = syscfing->addr;
			hijacksysc->opcodes[0] = syscfing->opcodes[0];
			hijacksysc->opcodes[1] = syscfing->opcodes[1];
			hash_insert(hijacksyscalls, key, KEYSIZE, hijacksysc);
		}
		tmp = tmp->next;
	}
	printf("-------------------------------------------------------------------------------\n");
	printf("[+] Begin : Syscall\n\n");
	
	if(hijacksyscalls->first != NULL){
		printf("LIST OF HIJACK SYSCALLS\n");
		simpleViewSyscalls(hijacksyscalls);
		printf("\n");
	}
	else
		printf("NO HIJACK SYSCALL\n\n");

	printf("[+] End : Syscall\n");
	printf("-------------------------------------------------------------------------------\n\n");
	
	hash_delete(syscallsmemory);
	hash_delete(syscallsfingerprints);
	hash_delete(hijacksyscalls);
}
